/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package ScheduleUtil;

import InitialSchedules.*;
import jdistlib.rng.RandomEngine;
import org.chocosolver.solver.Solution;



/**
 *
 * @author frabe
 */
public class SchedulesArray {
    
    protected boolean[][] initialPartTimeSchedules;
    protected boolean[][] initialFullTimeSchedules;
    protected boolean[][] partTimeSchedules;
    protected boolean[][] fullTimeSchedules;
    protected int[] employeesPerWorkPeriod;
    protected int[] workingPeriodsPerPartTimeEmployees;
    protected int[] workingPeriodsPerFullTimeEmployees;
    protected boolean[] isPartTimeEmployeeActive;
    protected boolean[] isFullTimeEmployeeActive;
    protected int maxPartTimeEmployee;
    protected int maxFullTimeEmployee;
    protected int workPeriodsPerSchedule;
    protected int workPeriodsPerDay;
    protected int daysPerSchedule;
    protected ModelInitialSchedules myModelInitialSchedules;
    
    protected SchedulesArray() {
        
    }
    
    // Constructeur Copie
    protected SchedulesArray(SchedulesArray myScheduleArray) {
        
        this.maxPartTimeEmployee = myScheduleArray.maxPartTimeEmployee;
        this.maxFullTimeEmployee = myScheduleArray.maxFullTimeEmployee;
        this.workPeriodsPerSchedule = myScheduleArray.workPeriodsPerSchedule;
        this.workPeriodsPerDay = myScheduleArray.workPeriodsPerDay;
        this.daysPerSchedule = myScheduleArray.daysPerSchedule;
        this.myModelInitialSchedules = myScheduleArray.myModelInitialSchedules;
        
        this.isPartTimeEmployeeActive = new  boolean[this.maxPartTimeEmployee];
        for (int employee = 0; employee < this.maxPartTimeEmployee; employee++) {
            this.isPartTimeEmployeeActive[employee] = myScheduleArray.isPartTimeEmployeeActive[employee];
        }
        
        this.isFullTimeEmployeeActive = new  boolean[this.maxFullTimeEmployee];
        for (int employee = 0; employee < this.maxFullTimeEmployee; employee++) {
            this.isFullTimeEmployeeActive[employee] = myScheduleArray.isFullTimeEmployeeActive[employee];
        }
        
        this.partTimeSchedules = new boolean[this.maxPartTimeEmployee][this.workPeriodsPerSchedule];
        this.fullTimeSchedules = new boolean[this.maxFullTimeEmployee][this.workPeriodsPerSchedule];
        this.initialPartTimeSchedules = new boolean[this.maxPartTimeEmployee][this.workPeriodsPerSchedule];
        this.initialFullTimeSchedules = new boolean[this.maxFullTimeEmployee][this.workPeriodsPerSchedule];
        for (int workPeriod = 0; workPeriod < this.workPeriodsPerSchedule; workPeriod++) {
            for (int employee = 0; employee < this.maxPartTimeEmployee; employee++) {
                this.partTimeSchedules[employee][workPeriod] = myScheduleArray.partTimeSchedules[employee][workPeriod];
                this.initialPartTimeSchedules[employee][workPeriod] = myScheduleArray.initialPartTimeSchedules[employee][workPeriod];
            }
        }

        for (int workPeriod = 0; workPeriod < this.workPeriodsPerSchedule; workPeriod++) {
            for (int employee = 0; employee < this.maxFullTimeEmployee; employee++) {
                this.fullTimeSchedules[employee][workPeriod] = myScheduleArray.fullTimeSchedules[employee][workPeriod];
                this.initialFullTimeSchedules[employee][workPeriod] = myScheduleArray.initialFullTimeSchedules[employee][workPeriod];
            }
        }
        
        this.employeesPerWorkPeriod = new int[this.workPeriodsPerSchedule];
        this.workingPeriodsPerPartTimeEmployees = new int[this.maxPartTimeEmployee];
        this.workingPeriodsPerFullTimeEmployees = new int[this.maxFullTimeEmployee];
        
        updateAllEmployeesPerWorkPeriod();
        updateAllWorkingPeriodsPerPartTimeEmployees();
        updateAllWorkingPeriodsPerFullTimeEmployees();
        
    }
    
    public SchedulesArray(ModelInitialSchedules m, Solution s) {
        this.myModelInitialSchedules = m;
        Solution mySolution = s;
        this.maxPartTimeEmployee = myModelInitialSchedules.maxPartTimeEmployee;
        this.maxFullTimeEmployee = myModelInitialSchedules.maxFullTimeEmployee;
        this.workPeriodsPerSchedule = myModelInitialSchedules.myScheduleParameters.workPeriodsPerSchedule;
        this.workPeriodsPerDay = myModelInitialSchedules.myScheduleParameters.workPeriodsPerDay;
        this.daysPerSchedule = myModelInitialSchedules.myScheduleParameters.daysPerSchedule;
        
        this.isPartTimeEmployeeActive = new  boolean[this.maxPartTimeEmployee];
        for (int employee = 0; employee < this.maxPartTimeEmployee; employee++) {
            if (mySolution.getIntVal(myModelInitialSchedules.workingPeriodsPerPartTimeEmployees[employee]) > 0){
                this.isPartTimeEmployeeActive[employee] = true;
            }
        }
        
        this.isFullTimeEmployeeActive = new  boolean[this.maxFullTimeEmployee];
        for (int employee = 0; employee < this.maxFullTimeEmployee; employee++) {
            if (mySolution.getIntVal(myModelInitialSchedules.workingPeriodsPerFullTimeEmployees[employee]) > 0){
                this.isFullTimeEmployeeActive[employee] = true;
            }
        }
        
        this.partTimeSchedules = new boolean[this.maxPartTimeEmployee][this.workPeriodsPerSchedule];
        this.fullTimeSchedules = new boolean[this.maxFullTimeEmployee][this.workPeriodsPerSchedule];
        this.initialPartTimeSchedules = new boolean[this.maxPartTimeEmployee][this.workPeriodsPerSchedule];
        this.initialFullTimeSchedules = new boolean[this.maxFullTimeEmployee][this.workPeriodsPerSchedule];
        for (int workPeriod = 0; workPeriod < this.workPeriodsPerSchedule; workPeriod++) {
            for (int employee = 0; employee < this.maxPartTimeEmployee; employee++) {
                this.partTimeSchedules[employee][workPeriod] = mySolution.getIntVal(myModelInitialSchedules.workPeriodsSchedulesOfPartTimeEmployees[employee][workPeriod]) == 1;
                this.initialPartTimeSchedules[employee][workPeriod] = mySolution.getIntVal(myModelInitialSchedules.workPeriodsSchedulesOfPartTimeEmployees[employee][workPeriod]) == 1;
            }
        }

        for (int workPeriod = 0; workPeriod < this.workPeriodsPerSchedule; workPeriod++) {
            for (int employee = 0; employee < this.maxFullTimeEmployee; employee++) {
                this.fullTimeSchedules[employee][workPeriod] = mySolution.getIntVal(myModelInitialSchedules.workPeriodsSchedulesOfFullTimeEmployees[employee][workPeriod]) == 1;
                this.initialFullTimeSchedules[employee][workPeriod] = mySolution.getIntVal(myModelInitialSchedules.workPeriodsSchedulesOfFullTimeEmployees[employee][workPeriod]) == 1;
            }
        }
        
        this.employeesPerWorkPeriod = new int[this.workPeriodsPerSchedule];
        this.workingPeriodsPerPartTimeEmployees = new int[this.maxPartTimeEmployee];
        this.workingPeriodsPerFullTimeEmployees = new int[this.maxFullTimeEmployee];
        
        updateAllEmployeesPerWorkPeriod();
        updateAllWorkingPeriodsPerPartTimeEmployees();
        updateAllWorkingPeriodsPerFullTimeEmployees();

    }
    
    protected void updateAllEmployeesPerWorkPeriod() {
        
        for (int workPeriod = 0; workPeriod < this.workPeriodsPerSchedule; workPeriod++) {
            updateEmployeesPerWorkPeriod(workPeriod);
        }
    }
    
    protected void updateEmployeesPerWorkPeriod(int workPeriod) {
        this.employeesPerWorkPeriod[workPeriod] = 0;
        for (int employee = 0; employee < this.maxPartTimeEmployee; employee++) {
            if (this.partTimeSchedules[employee][workPeriod]) {this.employeesPerWorkPeriod[workPeriod] += 1;}
        }
        for (int employee = 0; employee < this.maxFullTimeEmployee; employee++) {
            if (this.fullTimeSchedules[employee][workPeriod]) {this.employeesPerWorkPeriod[workPeriod] += 1;}
        }
    }
    
    protected void updateAllWorkingPeriodsPerPartTimeEmployees() {
        for (int employee = 0; employee < this.maxPartTimeEmployee; employee++) {
            updateWorkingPeriodsPerPartTimeEmployees(employee);
        }
    }
    
    protected void updateWorkingPeriodsPerPartTimeEmployees(int employee) {
        this.workingPeriodsPerPartTimeEmployees[employee] = 0;
        for (int workPeriod = 0; workPeriod < this.workPeriodsPerSchedule; workPeriod++) {
            if (this.partTimeSchedules[employee][workPeriod]) {this.workingPeriodsPerPartTimeEmployees[employee] += 1;}
        }
    }
    
    protected void updateAllWorkingPeriodsPerFullTimeEmployees() {
        for (int employee = 0; employee < this.maxFullTimeEmployee; employee++) {
            updateWorkingPeriodsPerFullTimeEmployees(employee);
        }
    }
    
    protected void updateWorkingPeriodsPerFullTimeEmployees(int employee) {
        this.workingPeriodsPerFullTimeEmployees[employee] = 0;
        for (int workPeriod = 0; workPeriod < this.workPeriodsPerSchedule; workPeriod++) {
            if (this.fullTimeSchedules[employee][workPeriod]) {this.workingPeriodsPerFullTimeEmployees[employee] += 1;}
        }
    }
    
    public int getMaxFullTimeEmployee() {
        return maxFullTimeEmployee;
    }

    public int getMaxPartTimeEmployee() {
        return maxPartTimeEmployee;
    }

    public int getWorkPeriodsPerSchedule() {
        return workPeriodsPerSchedule;
    }
    
    public int getDaysPerSchedule() {
        return daysPerSchedule;
    }
    
    public int getWorkPeriodPerDay() {
        return workPeriodsPerDay;
    }

    public boolean[][] getFullTimeSchedules() {
        return fullTimeSchedules;
    }

    public boolean[][] getPartTimeSchedules() {
        return partTimeSchedules;
    }
    
    public boolean isPartTimeEmployeeWorking(int employee, int workPeriod) {
        return partTimeSchedules[employee][workPeriod] == true;
    }
    
    public boolean isFullTimeEmployeeWorking(int employee, int workPeriod) {
        return fullTimeSchedules[employee][workPeriod] == true;
    }
    
    public boolean isPartTimeEmployeeActive(int employee) {
        return isPartTimeEmployeeActive[employee] == true;
    }
    
    public boolean isFullTimeEmployeeActive(int employee) {
        return isFullTimeEmployeeActive[employee] == true;
    }
    
    public int getRequiredWorkForce(int workPeriod) {
        return myModelInitialSchedules.myScheduleParameters.requiredWorkforce[workPeriod];
    }
    
    public int getEmployeesPerWorkPeriod(int workPeriod) {
        return employeesPerWorkPeriod[workPeriod];
    }
    
    public int getWorkingPeriodsPerPartTimeEmployees(int employee) {
        return workingPeriodsPerPartTimeEmployees[employee];
    }
    
    public int getWorkingPeriodsPerFullTimeEmployees(int employee) {
        return workingPeriodsPerFullTimeEmployees[employee];
    }
}
