/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package SchedulesRecovery;

import AbsenceSchedules.AbsenceSchedulesArray;


/**
 *
 * @author frabe
 */
public class RecoveredSchedulesArray extends AbsenceSchedulesArray{
    
    public RecoveredSchedulesArray(AbsenceSchedulesArray myAbsenceScheduleArray){
        super(myAbsenceScheduleArray);
    }
    
    public void recoverAbsenceScheduleOptimally() {
        for (int workPeriod = 0; workPeriod < this.workPeriodsPerSchedule; workPeriod++) {
            boolean noEmployeeAvailable = false;
            while (!noEmployeeAvailable && getEmployeesPerWorkPeriod(workPeriod) < getRequiredWorkForce(workPeriod)) {
                boolean absenceRecoveryDone = false;
                int employee = 0;
                while (!absenceRecoveryDone && employee < this.maxFullTimeEmployee) {
                    if (isFullTimeEmployeeAvailableForAbsenceRecovering(employee, workPeriod) && !isFullTimeEmployeeWorking(employee, workPeriod)){
                        fullTimeSchedules[employee][workPeriod] = true;
                        updateEmployeesPerWorkPeriod(workPeriod);
                        updateWorkingPeriodsPerFullTimeEmployees(employee);
                        absenceRecoveryDone = true;
                    }
                    employee++;
                }
                employee = 0;
                while (!absenceRecoveryDone && employee < this.maxPartTimeEmployee) {
                    if (isPartTimeEmployeeAvailableForAbsenceRecovering(employee, workPeriod) && !isPartTimeEmployeeWorking(employee, workPeriod)){
                        partTimeSchedules[employee][workPeriod] = true;
                        updateEmployeesPerWorkPeriod(workPeriod);
                        updateWorkingPeriodsPerPartTimeEmployees(employee);
                        absenceRecoveryDone = true;
                    }
                    employee++;
                }
                if (!absenceRecoveryDone) {
                    noEmployeeAvailable = true;
                }
            }
            
        }
        
    }
    
    private boolean isPartTimeEmployeeAvailableForAbsenceRecovering(int employee, int workPeriod){
        return isPartTimeEmployeeActive(employee) && !isPartTimeEmployeeAbsent(employee, workPeriod) && isValidPartTimeEmployeeWorkPeriod(employee,workPeriod);
    }
    
    private boolean isValidPartTimeEmployeeWorkPeriod(int employee, int workPeriod){
        return isValidWorkingPeriodsOfPartTimeEmployee(employee) &&
                isValidConsecutiveWorkingPeriodsOfPartTimeEmployee(employee, workPeriod) && 
                isValidConsecutiveNonWorkingPeriodsBetweenShiftWorksOfPartTimeEmployee(employee, workPeriod);
    }
    
    private boolean isValidWorkingPeriodsOfPartTimeEmployee(int employee){
        return this.workingPeriodsPerPartTimeEmployees[employee] < myModelInitialSchedules.myScheduleParameters.maxWorkingPeriodsOfPartTimeEmployeesPerSchedule;
    }

    private boolean isValidConsecutiveWorkingPeriodsOfPartTimeEmployee(int employee, int workPeriod){
        int consecutiveWorkingPeriods = 1;
        int compteurWorkPeriod = workPeriod - 1;
        while ( compteurWorkPeriod >= 0 && isPartTimeEmployeeWorking(employee, compteurWorkPeriod) ){
            consecutiveWorkingPeriods += 1;
            compteurWorkPeriod--;
        }
        compteurWorkPeriod = workPeriod + 1;
        while ( compteurWorkPeriod < this.workPeriodsPerSchedule && isPartTimeEmployeeWorking(employee, compteurWorkPeriod) ){
            consecutiveWorkingPeriods += 1;
            compteurWorkPeriod++;
        }
        return consecutiveWorkingPeriods <= myModelInitialSchedules.myScheduleParameters.maxConsecutiveWorkingPeriodsOfPartTimeEmployeesPerShiftWork;
    }

    private boolean isValidConsecutiveNonWorkingPeriodsBetweenShiftWorksOfPartTimeEmployee(int employee, int workPeriod){
        int consecutivePreviousNonWorkingPeriods = 0;
        int compteurWorkPeriod = workPeriod - 1;
        while ( compteurWorkPeriod >= 0 && !isPartTimeEmployeeWorking(employee, compteurWorkPeriod) ){
            consecutivePreviousNonWorkingPeriods += 1;
            compteurWorkPeriod--;
        }
        boolean validConsecutivePreviousNonWorkingPeriods = false;
        if (compteurWorkPeriod == -1) {
            validConsecutivePreviousNonWorkingPeriods = true;
        } else if (consecutivePreviousNonWorkingPeriods == 0) {
            validConsecutivePreviousNonWorkingPeriods = true;
        } else if (consecutivePreviousNonWorkingPeriods >= myModelInitialSchedules.myScheduleParameters.minConsecutiveNonWorkingPeriodsBetweenShiftWorksOfPartTimeEmployees) {
            validConsecutivePreviousNonWorkingPeriods = true;
        }
        int consecutiveNextNonWorkingPeriods = 0;
        compteurWorkPeriod = workPeriod + 1;
        while ( compteurWorkPeriod < this.workPeriodsPerSchedule && !isPartTimeEmployeeWorking(employee, compteurWorkPeriod) ){
            consecutiveNextNonWorkingPeriods += 1;
            compteurWorkPeriod++;
        }
        boolean validConsecutiveNextNonWorkingPeriods = false;
        if (compteurWorkPeriod == workPeriodsPerSchedule) {
            validConsecutiveNextNonWorkingPeriods = true;
        } else if (consecutiveNextNonWorkingPeriods == 0) {
            validConsecutiveNextNonWorkingPeriods = true;
        } else if (consecutiveNextNonWorkingPeriods >= myModelInitialSchedules.myScheduleParameters.minConsecutiveNonWorkingPeriodsBetweenShiftWorksOfPartTimeEmployees) {
            validConsecutiveNextNonWorkingPeriods = true;
        }
        return validConsecutivePreviousNonWorkingPeriods && validConsecutiveNextNonWorkingPeriods;
    }
    
    private boolean isFullTimeEmployeeAvailableForAbsenceRecovering(int employee, int workPeriod){
        return isFullTimeEmployeeActive(employee) && !isFullTimeEmployeeAbsent(employee, workPeriod) && isValidFullTimeEmployeeWorkPeriod(employee,workPeriod);
    }
    
    private boolean isValidFullTimeEmployeeWorkPeriod(int employee, int workPeriod){
        return isValidWorkingPeriodsOfFullTimeEmployee(employee) &&
                isValidConsecutiveWorkingPeriodsOfFullTimeEmployee(employee, workPeriod) && 
                isValidConsecutiveNonWorkingPeriodsBetweenShiftWorksOfFullTimeEmployee(employee, workPeriod);
    }
    
    private boolean isValidWorkingPeriodsOfFullTimeEmployee(int employee){
        return this.workingPeriodsPerFullTimeEmployees[employee] < myModelInitialSchedules.myScheduleParameters.maxWorkingPeriodsOfFullTimeEmployeesPerSchedule;
    }
    

    private boolean isValidConsecutiveWorkingPeriodsOfFullTimeEmployee(int employee, int workPeriod){
        int consecutiveWorkingPeriods = 1;
        int compteurWorkPeriod = workPeriod - 1;
        while ( compteurWorkPeriod >= 0 && isFullTimeEmployeeWorking(employee, compteurWorkPeriod) ){
            consecutiveWorkingPeriods += 1;
            compteurWorkPeriod--;
        }
        compteurWorkPeriod = workPeriod + 1;
        while ( compteurWorkPeriod < this.workPeriodsPerSchedule && isFullTimeEmployeeWorking(employee, compteurWorkPeriod) ){
            consecutiveWorkingPeriods += 1;
            compteurWorkPeriod++;
        }
        return consecutiveWorkingPeriods <= myModelInitialSchedules.myScheduleParameters.maxConsecutiveWorkingPeriodsOfFullTimeEmployeesPerShiftWork;
    }

    private boolean isValidConsecutiveNonWorkingPeriodsBetweenShiftWorksOfFullTimeEmployee(int employee, int workPeriod){
        int consecutivePreviousNonWorkingPeriods = 0;
        int compteurWorkPeriod = workPeriod - 1;
        while ( compteurWorkPeriod >= 0 && !isFullTimeEmployeeWorking(employee, compteurWorkPeriod)){
            consecutivePreviousNonWorkingPeriods += 1;
            compteurWorkPeriod--;
        }
        boolean validConsecutivePreviousNonWorkingPeriods = false;
        if (compteurWorkPeriod == -1) {
            validConsecutivePreviousNonWorkingPeriods = true;
        } else if (consecutivePreviousNonWorkingPeriods == 0) {
            validConsecutivePreviousNonWorkingPeriods = true;
        } else if (consecutivePreviousNonWorkingPeriods >= myModelInitialSchedules.myScheduleParameters.minConsecutiveNonWorkingPeriodsBetweenShiftWorksOfFullTimeEmployees) {
            validConsecutivePreviousNonWorkingPeriods = true;
        }
        int consecutiveNextNonWorkingPeriods = 0;
        compteurWorkPeriod = workPeriod + 1;
        while ( compteurWorkPeriod < this.workPeriodsPerSchedule && !isFullTimeEmployeeWorking(employee, compteurWorkPeriod) ){
            consecutiveNextNonWorkingPeriods += 1;
            compteurWorkPeriod++;
        }
        boolean validConsecutiveNextNonWorkingPeriods = false;
        if (compteurWorkPeriod == workPeriodsPerSchedule) {
            validConsecutiveNextNonWorkingPeriods = true;
        } else if (consecutiveNextNonWorkingPeriods == 0) {
            validConsecutiveNextNonWorkingPeriods = true;
        } else if (consecutiveNextNonWorkingPeriods >= myModelInitialSchedules.myScheduleParameters.minConsecutiveNonWorkingPeriodsBetweenShiftWorksOfFullTimeEmployees) {
            validConsecutiveNextNonWorkingPeriods = true;
        }
        return validConsecutivePreviousNonWorkingPeriods && validConsecutiveNextNonWorkingPeriods;
    }
}
